# encoding: utf-8

"""Gherkin step implementations for text-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, when, then

from pptx import Presentation
from pptx.enum.text import PP_ALIGN
from pptx.util import Emu

from helpers import test_pptx


# given ===================================================


@given("a _Paragraph object as paragraph")
def given_a_Paragraph_object_as_paragraph(context):
    prs = Presentation(test_pptx("txt-text"))
    context.paragraph = prs.slides[0].shapes[0].text_frame.paragraphs[0]


@given("a _Paragraph object containing {value} as paragraph")
def given_a_Paragraph_object_containing_value_as_paragraph(context, value):
    prs = Presentation(test_pptx("txt-text"))
    paragraph_idx = {"abc": 0, "a\vb\vc": 1}[eval(value)]
    context.paragraph = prs.slides[0].shapes[1].text_frame.paragraphs[paragraph_idx]


@given("a paragraph having line spacing of {setting}")
def given_a_paragraph_having_line_spacing_of_setting(context, setting):
    paragraph_idx = {"no explicit setting": 0, "1.5 lines": 1, "20 pt": 2}[setting]
    prs = Presentation(test_pptx("txt-paragraph-spacing"))
    text_frame = prs.slides[2].shapes[0].text_frame
    context.paragraph = text_frame.paragraphs[paragraph_idx]


@given("a paragraph having space {before_after} of {setting}")
def given_a_paragraph_having_space_before_after_of_setting(
    context, before_after, setting
):
    slide_idx = {"before": 0, "after": 1}[before_after]
    paragraph_idx = {"no explicit setting": 0, "6 pt": 1}[setting]
    prs = Presentation(test_pptx("txt-paragraph-spacing"))
    text_frame = prs.slides[slide_idx].shapes[0].text_frame
    context.paragraph = text_frame.paragraphs[paragraph_idx]


@given("a _Run object as run")
def given_a_Run_object_as_run(context):
    prs = Presentation(test_pptx("txt-text"))
    context.run = prs.slides[0].shapes[0].text_frame.paragraphs[0].runs[0]


@given("a _Run object containing text as run")
def given_a_Run_object_containing_text_as_run(context):
    prs = Presentation(test_pptx("txt-text"))
    context.run = prs.slides[0].shapes[0].text_frame.paragraphs[0].runs[0]


@given("a text run")
def given_a_text_run(context):
    prs = Presentation(test_pptx("txt-text"))
    context.run = prs.slides[0].shapes[0].text_frame.paragraphs[0].runs[0]


@given("a text run in a table cell")
def given_a_text_run_in_a_table_cell(context):
    prs = Presentation(test_pptx("txt-text"))
    cell = prs.slides[1].shapes[0].table.cell(0, 0)
    context.run = cell.text_frame.paragraphs[0].runs[0]


@given("a text run having a hyperlink")
def given_a_text_run_having_a_hyperlink(context):
    prs = Presentation(test_pptx("txt-text"))
    context.run = prs.slides[0].shapes[0].text_frame.paragraphs[1].runs[0]


# when ====================================================


@when("I assign a typeface name to the font")
def when_assign_typeface_name_to_font(context):
    context.font.name = "Verdana"


@when("I assign None to hyperlink.address")
def when_assign_None_to_hyperlink_address(context):
    context.run.hyperlink.address = None


@when("I assign paragraph.alignment = PP_ALIGN.CENTER")
def when_I_assign_paragraph_alignment_eq_center(context):
    context.paragraph.alignment = PP_ALIGN.CENTER


@when("I assign paragraph.level = 1")
def when_I_assign_paragraph_leve_eq_1(context):
    context.paragraph.level = 1


@when("I assign paragraph.text = {value}")
def when_I_assign_paragraph_text_eq_value(context, value):
    context.paragraph.text = eval(value)


@when("I assign run.text = {value}")
def when_I_assign_run_text_eq_value(context, value):
    context.run.text = eval(value)


@when("I assign {value_str} to paragraph.line_spacing")
def when_I_assign_value_to_paragraph_line_spacing(context, value_str):
    value = {
        "1.5": 1.5,
        "2.0": 2.0,
        "254000": Emu(254000),
        "304800": Emu(304800),
        "None": None,
    }[value_str]
    paragraph = context.paragraph
    paragraph.line_spacing = value


@when("I assign {value_str} to paragraph.space_{before_after}")
def when_I_assign_value_to_paragraph_space_before_after(
    context, value_str, before_after
):
    value = {"76200": 76200, "38100": 38100, "None": None}[value_str]
    attr_name = {"before": "space_before", "after": "space_after"}[before_after]
    paragraph = context.paragraph
    setattr(paragraph, attr_name, value)


@when("I set the hyperlink address")
def when_set_hyperlink_address(context):
    context.run_text = "python-pptx @ GitHub"
    context.address = "https://github.com/scanny/python-pptx"

    run = context.run
    run.text = context.run_text
    hlink = run.hyperlink
    hlink.address = context.address


# then ====================================================


@then("paragraph.alignment == PP_ALIGN.CENTER")
def then_paragraph_alignment_eq_center(context):
    actual, expected = context.paragraph.alignment, PP_ALIGN.CENTER
    assert actual == expected, "paragraph.alignment == %s" % actual


@then("paragraph.level == 1")
def then_paragraph_level_eq_1(context):
    actual, expected = context.paragraph.level, 1
    assert actual == expected, "paragraph.level == %s" % actual


@then("paragraph.line_spacing is {value_str}")
def then_paragraph_line_spacing_is_value(context, value_str):
    value = {
        "None": None,
        "1.0": 1.0,
        "1.5": 1.5,
        "2.0": 2.0,
        "254000": 254000,
        "304800": 304800,
    }[value_str]
    paragraph = context.paragraph
    assert paragraph.line_spacing == value


@then("paragraph.line_spacing.pt {result}")
def then_paragraph_line_spacing_pt_result(context, result):
    value, exception = {
        "raises AttributeError": (None, AttributeError),
        "is 20.0": (20.0, None),
        "is 24.0": (24.0, None),
    }[result]
    line_spacing = context.paragraph.line_spacing
    if value is not None:
        assert line_spacing.pt == value
    if exception is not None:
        try:
            line_spacing.pt
            raise AssertionError("did not raise")
        except exception:
            pass


@then("paragraph.space_{before_after} is {value_str}")
def then_paragraph_space_before_is_value(context, before_after, value_str):
    attr_name = {"before": "space_before", "after": "space_after"}[before_after]
    value = None if value_str == "None" else int(value_str)
    paragraph = context.paragraph
    assert getattr(paragraph, attr_name) == value


@then("paragraph.space_{before_after}.pt {result}")
def then_paragraph_space_before_pt_is_value(context, before_after, result):
    attr_name = {"before": "space_before", "after": "space_after"}[before_after]
    value, exception = {
        "raises AttributeError": (None, AttributeError),
        "== 6.0": (6.0, None),
    }[result]
    space_before_after = getattr(context.paragraph, attr_name)
    if value is not None:
        assert space_before_after.pt == value
    if exception is not None:
        try:
            space_before_after.pt
            raise AssertionError("did not raise")
        except exception:
            pass


@then("paragraph.text == {value}")
def then_paragraph_text_eq_value(context, value):
    actual, expected = context.paragraph.text, eval(value)
    assert actual == expected, 'paragraph.text == "%s"' % actual


@then("paragraph.text matches the assigned string")
def then_paragraph_text_matches_the_assigned_string(context):
    paragraph = context.paragraph
    assert paragraph.text == " Boo Far \n Faz Foo "


@then("run.text == {value}")
def then_run_text_is_value(context, value):
    actual, expected = context.run.text, eval(value)
    assert actual == expected, "run.text == %s" % (actual,)


@then("run.text is a hyperlink")
def then_run_text_is_a_hyperlink(context):
    run = context.run
    hlink = run.hyperlink
    assert run.text == context.run_text
    assert hlink.address == context.address


@then("run.text is not a hyperlink")
def then_run_text_is_not_a_hyperlink(context):
    hlink = context.run.hyperlink
    assert hlink.address is None


@then("the font name matches the typeface I set")
def then_font_name_matches_typeface_I_set(context):
    assert context.font.name == "Verdana"



# encoding: utf-8

"""
Series-related oxml objects.
"""

from __future__ import absolute_import, print_function, unicode_literals

from .datalabel import CT_DLbls
from ..simpletypes import XsdUnsignedInt
from ..xmlchemy import (
    BaseOxmlElement,
    OneAndOnlyOne,
    OxmlElement,
    RequiredAttribute,
    ZeroOrMore,
    ZeroOrOne,
)


class CT_AxDataSource(BaseOxmlElement):
    """
    ``<c:cat>`` custom element class used in category charts to specify
    category labels and hierarchy.
    """

    multiLvlStrRef = ZeroOrOne("c:multiLvlStrRef", successors=())

    @property
    def lvls(self):
        """
        Return a list containing the `c:lvl` descendent elements in document
        order. These will only be present when the required single child
        is a `c:multiLvlStrRef` element. Returns an empty list when no
        `c:lvl` descendent elements are present.
        """
        return self.xpath(".//c:lvl")


class CT_DPt(BaseOxmlElement):
    """
    ``<c:dPt>`` custom element class, containing visual properties for a data
    point.
    """

    _tag_seq = (
        "c:idx",
        "c:invertIfNegative",
        "c:marker",
        "c:bubble3D",
        "c:explosion",
        "c:spPr",
        "c:pictureOptions",
        "c:extLst",
    )
    idx = OneAndOnlyOne("c:idx")
    marker = ZeroOrOne("c:marker", successors=_tag_seq[3:])
    spPr = ZeroOrOne("c:spPr", successors=_tag_seq[6:])
    del _tag_seq

    @classmethod
    def new_dPt(cls):
        """
        Return a newly created "loose" `c:dPt` element containing its default
        subtree.
        """
        dPt = OxmlElement("c:dPt")
        dPt.append(OxmlElement("c:idx"))
        return dPt


class CT_Lvl(BaseOxmlElement):
    """
    ``<c:lvl>`` custom element class used in multi-level categories to
    specify a level of hierarchy.
    """

    pt = ZeroOrMore("c:pt", successors=())


class CT_NumDataSource(BaseOxmlElement):
    """
    ``<c:yVal>`` custom element class used in XY and bubble charts, and
    perhaps others.
    """

    numRef = OneAndOnlyOne("c:numRef")

    @property
    def ptCount_val(self):
        """
        Return the value of `./c:numRef/c:numCache/c:ptCount/@val`,
        specifying how many `c:pt` elements are in this numeric data cache.
        Returns 0 if no `c:ptCount` element is present, as this is the least
        disruptive way to degrade when no cached point data is available.
        This situation is not expected, but is valid according to the schema.
        """
        results = self.xpath(".//c:ptCount/@val")
        return int(results[0]) if results else 0

    def pt_v(self, idx):
        """
        Return the Y value for data point *idx* in this cache, or None if no
        value is present for that data point.
        """
        results = self.xpath(".//c:pt[@idx=%d]" % idx)
        return results[0].value if results else None


class CT_SeriesComposite(BaseOxmlElement):
    """
    ``<c:ser>`` custom element class. Note there are several different series
    element types in the schema, such as ``CT_LineSer`` and ``CT_BarSer``,
    but they all share the same tag name. This class acts as a composite and
    depends on the caller not to do anything invalid for a series belonging
    to a particular plot type.
    """

    _tag_seq = (
        "c:idx",
        "c:order",
        "c:tx",
        "c:spPr",
        "c:invertIfNegative",
        "c:pictureOptions",
        "c:marker",
        "c:explosion",
        "c:dPt",
        "c:dLbls",
        "c:trendline",
        "c:errBars",
        "c:cat",
        "c:val",
        "c:xVal",
        "c:yVal",
        "c:shape",
        "c:smooth",
        "c:bubbleSize",
        "c:bubble3D",
        "c:extLst",
    )
    idx = OneAndOnlyOne("c:idx")
    order = OneAndOnlyOne("c:order")
    tx = ZeroOrOne("c:tx", successors=_tag_seq[3:])
    spPr = ZeroOrOne("c:spPr", successors=_tag_seq[4:])
    invertIfNegative = ZeroOrOne("c:invertIfNegative", successors=_tag_seq[5:])
    marker = ZeroOrOne("c:marker", successors=_tag_seq[7:])
    dPt = ZeroOrMore("c:dPt", successors=_tag_seq[9:])
    dLbls = ZeroOrOne("c:dLbls", successors=_tag_seq[10:])
    cat = ZeroOrOne("c:cat", successors=_tag_seq[13:])
    val = ZeroOrOne("c:val", successors=_tag_seq[14:])
    xVal = ZeroOrOne("c:xVal", successors=_tag_seq[15:])
    yVal = ZeroOrOne("c:yVal", successors=_tag_seq[16:])
    smooth = ZeroOrOne("c:smooth", successors=_tag_seq[18:])
    bubbleSize = ZeroOrOne("c:bubbleSize", successors=_tag_seq[19:])
    del _tag_seq

    @property
    def bubbleSize_ptCount_val(self):
        """
        Return the number of bubble size values as reflected in the `val`
        attribute of `./c:bubbleSize//c:ptCount`, or 0 if not present.
        """
        vals = self.xpath("./c:bubbleSize//c:ptCount/@val")
        if not vals:
            return 0
        return int(vals[0])

    @property
    def cat_ptCount_val(self):
        """
        Return the number of categories as reflected in the `val` attribute
        of `./c:cat//c:ptCount`, or 0 if not present.
        """
        vals = self.xpath("./c:cat//c:ptCount/@val")
        if not vals:
            return 0
        return int(vals[0])

    def get_dLbl(self, idx):
        """
        Return the `c:dLbl` element representing the label for the data point
        at offset *idx* in this series, or |None| if not present.
        """
        dLbls = self.dLbls
        if dLbls is None:
            return None
        return dLbls.get_dLbl_for_point(idx)

    def get_or_add_dLbl(self, idx):
        """
        Return the `c:dLbl` element representing the label of the point at
        offset *idx* in this series, newly created if not yet present.
        """
        dLbls = self.get_or_add_dLbls()
        return dLbls.get_or_add_dLbl_for_point(idx)

    def get_or_add_dPt_for_point(self, idx):
        """
        Return the `c:dPt` child representing the visual properties of the
        data point at index *idx*.
        """
        matches = self.xpath('c:dPt[c:idx[@val="%d"]]' % idx)
        if matches:
            return matches[0]
        dPt = self._add_dPt()
        dPt.idx.val = idx
        return dPt

    @property
    def xVal_ptCount_val(self):
        """
        Return the number of X values as reflected in the `val` attribute of
        `./c:xVal//c:ptCount`, or 0 if not present.
        """
        vals = self.xpath("./c:xVal//c:ptCount/@val")
        if not vals:
            return 0
        return int(vals[0])

    @property
    def yVal_ptCount_val(self):
        """
        Return the number of Y values as reflected in the `val` attribute of
        `./c:yVal//c:ptCount`, or 0 if not present.
        """
        vals = self.xpath("./c:yVal//c:ptCount/@val")
        if not vals:
            return 0
        return int(vals[0])

    def _new_dLbls(self):
        """Override metaclass method that creates `c:dLbls` element."""
        return CT_DLbls.new_dLbls()

    def _new_dPt(self):
        """
        Overrides the metaclass generated method to get `c:dPt` with minimal
        subtree.
        """
        return CT_DPt.new_dPt()


class CT_StrVal_NumVal_Composite(BaseOxmlElement):
    """
    ``<c:pt>`` element, can be either CT_StrVal or CT_NumVal complex type.
    Using this class for both, differentiating as needed.
    """

    v = OneAndOnlyOne("c:v")
    idx = RequiredAttribute("idx", XsdUnsignedInt)

    @property
    def value(self):
        """
        The float value of the text in the required ``<c:v>`` child.
        """
        return float(self.v.text)



# encoding: utf-8

"""Initialization module for python-pptx package."""

__version__ = "0.6.20"


import pptx.exc as exceptions
import sys

sys.modules["pptx.exceptions"] = exceptions
del sys

from pptx.api import Presentation  # noqa

from pptx.opc.constants import CONTENT_TYPE as CT  # noqa: E402
from pptx.opc.package import PartFactory  # noqa: E402
from pptx.parts.chart import ChartPart  # noqa: E402
from pptx.parts.coreprops import CorePropertiesPart  # noqa: E402
from pptx.parts.image import ImagePart  # noqa: E402
from pptx.parts.media import MediaPart  # noqa: E402
from pptx.parts.presentation import PresentationPart  # noqa: E402
from pptx.parts.slide import (  # noqa: E402
    NotesMasterPart,
    NotesSlidePart,
    SlideLayoutPart,
    SlideMasterPart,
    SlidePart,
)

content_type_to_part_class_map = {
    CT.PML_PRESENTATION_MAIN: PresentationPart,
    CT.PML_PRES_MACRO_MAIN: PresentationPart,
    CT.PML_TEMPLATE_MAIN: PresentationPart,
    CT.PML_SLIDESHOW_MAIN: PresentationPart,
    CT.OPC_CORE_PROPERTIES: CorePropertiesPart,
    CT.PML_NOTES_MASTER: NotesMasterPart,
    CT.PML_NOTES_SLIDE: NotesSlidePart,
    CT.PML_SLIDE: SlidePart,
    CT.PML_SLIDE_LAYOUT: SlideLayoutPart,
    CT.PML_SLIDE_MASTER: SlideMasterPart,
    CT.DML_CHART: ChartPart,
    CT.BMP: ImagePart,
    CT.GIF: ImagePart,
    CT.JPEG: ImagePart,
    CT.MS_PHOTO: ImagePart,
    CT.PNG: ImagePart,
    CT.TIFF: ImagePart,
    CT.X_EMF: ImagePart,
    CT.X_WMF: ImagePart,
    CT.ASF: MediaPart,
    CT.AVI: MediaPart,
    CT.MOV: MediaPart,
    CT.MP4: MediaPart,
    CT.MPG: MediaPart,
    CT.MS_VIDEO: MediaPart,
    CT.SWF: MediaPart,
    CT.VIDEO: MediaPart,
    CT.WMV: MediaPart,
    CT.X_MS_VIDEO: MediaPart,
}

PartFactory.part_type_for.update(content_type_to_part_class_map)

del (
    ChartPart,
    CorePropertiesPart,
    ImagePart,
    MediaPart,
    SlidePart,
    SlideLayoutPart,
    SlideMasterPart,
    PresentationPart,
    CT,
    PartFactory,
)



# encoding: utf-8

"""Gherkin step implementations for ColorFormat-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, when, then

from pptx import Presentation
from pptx.dml.color import RGBColor
from pptx.enum.dml import MSO_THEME_COLOR

from helpers import test_pptx


# given ====================================================


@given("a ColorFormat object as color")
def given_a_ColorFormat_object_as_color(context):
    shape = Presentation(test_pptx("dml-fill")).slides[0].shapes[2]
    color = shape.fill.fore_color
    context.color = color


# when =====================================================


@when("I assign MSO_THEME_COLOR.ACCENT_6 to color.theme_color")
def when_assign_MSO_THEME_COLOR_ACCENT_6_to_color_theme_color(context):
    context.color.theme_color = MSO_THEME_COLOR.ACCENT_6


@when("I assign RGBColor(12, 34, 56) to color.rgb")
def when_I_assign_RGBColor_to_color_rgb(context):
    context.color.rgb = RGBColor(12, 34, 56)


@when("I assign 0.42 to color.brightness")
def when_I_assign_0_42_to_color_brightness(context):
    context.color.brightness = 0.42


# then =====================================================


@then("color.brightness is 0.42")
def then_color_brightness_is_value(context):
    brightness = context.color.brightness
    expected_value = 0.42
    assert brightness == expected_value, "expected %s, got %s" % (
        expected_value,
        brightness,
    )


@then("color.rgb is RGBColor(12, 34, 56)")
def then_color_rgb_is_RGBColor_12_34_56(context):
    rgb = context.color.rgb
    expected_value = RGBColor(12, 34, 56)
    assert rgb == expected_value, "expected %s, got %s" % (
        repr(expected_value),
        repr(rgb),
    )


@then("color.theme_color is MSO_THEME_COLOR.ACCENT_6")
def then_color_theme_color_is_MSO_THEME_COLOR_ACCENT_6(context):
    theme_color = context.color.theme_color
    expected_value = MSO_THEME_COLOR.ACCENT_6
    assert theme_color == expected_value, "expected %s, got %s" % (
        expected_value,
        theme_color,
    )



# encoding: utf-8

"""Category-related objects.

The |category.Categories| object is returned by ``Plot.categories`` and contains zero or
more |category.Category| objects, each representing one of the category labels
associated with the plot. Categories can be hierarchical, so there are members allowing
discovery of the depth of that hierarchy and providing means to navigate it.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.compat import Sequence


class Categories(Sequence):
    """
    A sequence of |category.Category| objects, each representing a category
    label on the chart. Provides properties for dealing with hierarchical
    categories.
    """

    def __init__(self, xChart):
        super(Categories, self).__init__()
        self._xChart = xChart

    def __getitem__(self, idx):
        pt = self._xChart.cat_pts[idx]
        return Category(pt, idx)

    def __iter__(self):
        cat_pts = self._xChart.cat_pts
        for idx, pt in enumerate(cat_pts):
            yield Category(pt, idx)

    def __len__(self):
        # a category can be "null", meaning the Excel cell for it is empty.
        # In this case, there is no c:pt element for it. The "empty" category
        # will, however, be accounted for in c:cat//c:ptCount/@val, which
        # reflects the true length of the categories collection.
        return self._xChart.cat_pt_count

    @property
    def depth(self):
        """
        Return an integer representing the number of hierarchical levels in
        this category collection. Returns 1 for non-hierarchical categories
        and 0 if no categories are present (generally meaning no series are
        present).
        """
        cat = self._xChart.cat
        if cat is None:
            return 0
        if cat.multiLvlStrRef is None:
            return 1
        return len(cat.lvls)

    @property
    def flattened_labels(self):
        """
        Return a sequence of tuples, each containing the flattened hierarchy
        of category labels for a leaf category. Each tuple is in parent ->
        child order, e.g. ``('US', 'CA', 'San Francisco')``, with the leaf
        category appearing last. If this categories collection is
        non-hierarchical, each tuple will contain only a leaf category label.
        If the plot has no series (and therefore no categories), an empty
        tuple is returned.
        """
        cat = self._xChart.cat
        if cat is None:
            return ()

        if cat.multiLvlStrRef is None:
            return tuple([(category.label,) for category in self])

        return tuple(
            [
                tuple([category.label for category in reversed(flat_cat)])
                for flat_cat in self._iter_flattened_categories()
            ]
        )

    @property
    def levels(self):
        """
        Return a sequence of |CategoryLevel| objects representing the
        hierarchy of this category collection. The sequence is empty when the
        category collection is not hierarchical, that is, contains only
        leaf-level categories. The levels are ordered from the leaf level to
        the root level; so the first level will contain the same categories
        as this category collection.
        """
        cat = self._xChart.cat
        if cat is None:
            return []
        return [CategoryLevel(lvl) for lvl in cat.lvls]

    def _iter_flattened_categories(self):
        """
        Generate a ``tuple`` object for each leaf category in this
        collection, containing the leaf category followed by its "parent"
        categories, e.g. ``('San Francisco', 'CA', 'USA'). Each tuple will be
        the same length as the number of levels (excepting certain edge
        cases which I believe always indicate a chart construction error).
        """
        levels = self.levels
        if not levels:
            return
        leaf_level, remaining_levels = levels[0], levels[1:]
        for category in leaf_level:
            yield self._parentage((category,), remaining_levels)

    def _parentage(self, categories, levels):
        """
        Return a tuple formed by recursively concatenating *categories* with
        its next ancestor from *levels*. The idx value of the first category
        in *categories* determines parentage in all levels. The returned
        sequence is in child -> parent order. A parent category is the
        Category object in a next level having the maximum idx value not
        exceeding that of the leaf category.
        """
        # exhausting levels is the expected recursion termination condition
        if not levels:
            return tuple(categories)

        # guard against edge case where next level is present but empty. That
        # situation is not prohibited for some reason.
        if not levels[0]:
            return tuple(categories)

        parent_level, remaining_levels = levels[0], levels[1:]
        leaf_node = categories[0]

        # Make the first parent the default. A possible edge case is where no
        # parent is defined for one or more leading values, e.g. idx > 0 for
        # the first parent.
        parent = parent_level[0]
        for category in parent_level:
            if category.idx > leaf_node.idx:
                break
            parent = category

        extended_categories = tuple(categories) + (parent,)
        return self._parentage(extended_categories, remaining_levels)


class Category(str):
    """
    An extension of `str` that provides the category label as its string
    value, and additional attributes representing other aspects of the
    category.
    """

    def __new__(cls, pt, *args):
        category_label = "" if pt is None else pt.v.text
        return str.__new__(cls, category_label)

    def __init__(self, pt, idx=None):
        """
        *idx* is a required attribute of a c:pt element, but must be
        specified when pt is None, as when a "placeholder" category is
        created to represent a missing c:pt element.
        """
        self._element = self._pt = pt
        self._idx = idx

    @property
    def idx(self):
        """
        Return an integer representing the index reference of this category.
        For a leaf node, the index identifies the category. For a parent (or
        other ancestor) category, the index specifies the first leaf category
        that ancestor encloses.
        """
        if self._pt is None:
            return self._idx
        return self._pt.idx

    @property
    def label(self):
        """
        Return the label of this category as a string.
        """
        return str(self)


class CategoryLevel(Sequence):
    """
    A sequence of |category.Category| objects representing a single level in
    a hierarchical category collection. This object is only used when the
    categories are hierarchical, meaning they have more than one level and
    higher level categories group those at lower levels.
    """

    def __init__(self, lvl):
        self._element = self._lvl = lvl

    def __getitem__(self, offset):
        return Category(self._lvl.pt_lst[offset])

    def __len__(self):
        return len(self._lvl.pt_lst)



# encoding: utf-8

"""
Enumerations used for specifying language.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from .base import (
    EnumMember,
    ReturnValueOnlyEnumMember,
    XmlEnumeration,
    XmlMappedEnumMember,
)


class MSO_LANGUAGE_ID(XmlEnumeration):
    """
    Specifies the language identifier.

    Example::

        from pptx.enum.lang import MSO_LANGUAGE_ID

        font.language_id = MSO_LANGUAGE_ID.POLISH
    """

    __ms_name__ = "MsoLanguageId"

    __url__ = "https://msdn.microsoft.com/en-us/library/office/ff862134.aspx"

    __members__ = (
        ReturnValueOnlyEnumMember(
            "MIXED", -2, "More than one language in specified range."
        ),
        EnumMember("NONE", 0, "No language specified."),
        XmlMappedEnumMember("AFRIKAANS", 1078, "af-ZA", "The Afrikaans language."),
        XmlMappedEnumMember("ALBANIAN", 1052, "sq-AL", "The Albanian language."),
        XmlMappedEnumMember("AMHARIC", 1118, "am-ET", "The Amharic language."),
        XmlMappedEnumMember("ARABIC", 1025, "ar-SA", "The Arabic language."),
        XmlMappedEnumMember(
            "ARABIC_ALGERIA", 5121, "ar-DZ", "The Arabic Algeria language."
        ),
        XmlMappedEnumMember(
            "ARABIC_BAHRAIN", 15361, "ar-BH", "The Arabic Bahrain language."
        ),
        XmlMappedEnumMember(
            "ARABIC_EGYPT", 3073, "ar-EG", "The Arabic Egypt language."
        ),
        XmlMappedEnumMember("ARABIC_IRAQ", 2049, "ar-IQ", "The Arabic Iraq language."),
        XmlMappedEnumMember(
            "ARABIC_JORDAN", 11265, "ar-JO", "The Arabic Jordan language."
        ),
        XmlMappedEnumMember(
            "ARABIC_KUWAIT", 13313, "ar-KW", "The Arabic Kuwait language."
        ),
        XmlMappedEnumMember(
            "ARABIC_LEBANON", 12289, "ar-LB", "The Arabic Lebanon language."
        ),
        XmlMappedEnumMember(
            "ARABIC_LIBYA", 4097, "ar-LY", "The Arabic Libya language."
        ),
        XmlMappedEnumMember(
            "ARABIC_MOROCCO", 6145, "ar-MA", "The Arabic Morocco language."
        ),
        XmlMappedEnumMember("ARABIC_OMAN", 8193, "ar-OM", "The Arabic Oman language."),
        XmlMappedEnumMember(
            "ARABIC_QATAR", 16385, "ar-QA", "The Arabic Qatar language."
        ),
        XmlMappedEnumMember(
            "ARABIC_SYRIA", 10241, "ar-SY", "The Arabic Syria language."
        ),
        XmlMappedEnumMember(
            "ARABIC_TUNISIA", 7169, "ar-TN", "The Arabic Tunisia language."
        ),
        XmlMappedEnumMember("ARABIC_UAE", 14337, "ar-AE", "The Arabic UAE language."),
        XmlMappedEnumMember(
            "ARABIC_YEMEN", 9217, "ar-YE", "The Arabic Yemen language."
        ),
        XmlMappedEnumMember("ARMENIAN", 1067, "hy-AM", "The Armenian language."),
        XmlMappedEnumMember("ASSAMESE", 1101, "as-IN", "The Assamese language."),
        XmlMappedEnumMember(
            "AZERI_CYRILLIC", 2092, "az-AZ", "The Azeri Cyrillic language."
        ),
        XmlMappedEnumMember(
            "AZERI_LATIN", 1068, "az-Latn-AZ", "The Azeri Latin language."
        ),
        XmlMappedEnumMember("BASQUE", 1069, "eu-ES", "The Basque language."),
        XmlMappedEnumMember(
            "BELGIAN_DUTCH", 2067, "nl-BE", "The Belgian Dutch language."
        ),
        XmlMappedEnumMember(
            "BELGIAN_FRENCH", 2060, "fr-BE", "The Belgian French language."
        ),
        XmlMappedEnumMember("BENGALI", 1093, "bn-IN", "The Bengali language."),
        XmlMappedEnumMember("BOSNIAN", 4122, "hr-BA", "The Bosnian language."),
        XmlMappedEnumMember(
            "BOSNIAN_BOSNIA_HERZEGOVINA_CYRILLIC",
            8218,
            "bs-BA",
            "The Bosni" "an Bosnia Herzegovina Cyrillic language.",
        ),
        XmlMappedEnumMember(
            "BOSNIAN_BOSNIA_HERZEGOVINA_LATIN",
            5146,
            "bs-Latn-BA",
            "The Bos" "nian Bosnia Herzegovina Latin language.",
        ),
        XmlMappedEnumMember(
            "BRAZILIAN_PORTUGUESE",
            1046,
            "pt-BR",
            "The Brazilian Portuguese" " language.",
        ),
        XmlMappedEnumMember("BULGARIAN", 1026, "bg-BG", "The Bulgarian language."),
        XmlMappedEnumMember("BURMESE", 1109, "my-MM", "The Burmese language."),
        XmlMappedEnumMember(
            "BYELORUSSIAN", 1059, "be-BY", "The Byelorussian language."
        ),
        XmlMappedEnumMember("CATALAN", 1027, "ca-ES", "The Catalan language."),
        XmlMappedEnumMember("CHEROKEE", 1116, "chr-US", "The Cherokee language."),
        XmlMappedEnumMember(
            "CHINESE_HONG_KONG_SAR",
            3076,
            "zh-HK",
            "The Chinese Hong Kong S" "AR language.",
        ),
        XmlMappedEnumMember(
            "CHINESE_MACAO_SAR", 5124, "zh-MO", "The Chinese Macao SAR langu" "age."
        ),
        XmlMappedEnumMember(
            "CHINESE_SINGAPORE", 4100, "zh-SG", "The Chinese Singapore langu" "age."
        ),
        XmlMappedEnumMember("CROATIAN", 1050, "hr-HR", "The Croatian language."),
        XmlMappedEnumMember("CZECH", 1029, "cs-CZ", "The Czech language."),
        XmlMappedEnumMember("DANISH", 1030, "da-DK", "The Danish language."),
        XmlMappedEnumMember("DIVEHI", 1125, "div-MV", "The Divehi language."),
        XmlMappedEnumMember("DUTCH", 1043, "nl-NL", "The Dutch language."),
        XmlMappedEnumMember("EDO", 1126, "bin-NG", "The Edo language."),
        XmlMappedEnumMember("ENGLISH_AUS", 3081, "en-AU", "The English AUS language."),
        XmlMappedEnumMember(
            "ENGLISH_BELIZE", 10249, "en-BZ", "The English Belize language."
        ),
        XmlMappedEnumMember(
            "ENGLISH_CANADIAN", 4105, "en-CA", "The English Canadian languag" "e."
        ),
        XmlMappedEnumMember(
            "ENGLISH_CARIBBEAN", 9225, "en-CB", "The English Caribbean langu" "age."
        ),
        XmlMappedEnumMember(
            "ENGLISH_INDONESIA", 14345, "en-ID", "The English Indonesia lang" "uage."
        ),
        XmlMappedEnumMember(
            "ENGLISH_IRELAND", 6153, "en-IE", "The English Ireland language."
        ),
        XmlMappedEnumMember(
            "ENGLISH_JAMAICA", 8201, "en-JA", "The English Jamaica language."
        ),
        XmlMappedEnumMember(
            "ENGLISH_NEW_ZEALAND", 5129, "en-NZ", "The English NewZealand la" "nguage."
        ),
        XmlMappedEnumMember(
            "ENGLISH_PHILIPPINES",
            13321,
            "en-PH",
            "The English Philippines " "language.",
        ),
        XmlMappedEnumMember(
            "ENGLISH_SOUTH_AFRICA",
            7177,
            "en-ZA",
            "The English South Africa" " language.",
        ),
        XmlMappedEnumMember(
            "ENGLISH_TRINIDAD_TOBAGO",
            11273,
            "en-TT",
            "The English Trinidad" " Tobago language.",
        ),
        XmlMappedEnumMember("ENGLISH_UK", 2057, "en-GB", "The English UK language."),
        XmlMappedEnumMember("ENGLISH_US", 1033, "en-US", "The English US language."),
        XmlMappedEnumMember(
            "ENGLISH_ZIMBABWE", 12297, "en-ZW", "The English Zimbabwe langua" "ge."
        ),
        XmlMappedEnumMember("ESTONIAN", 1061, "et-EE", "The Estonian language."),
        XmlMappedEnumMember("FAEROESE", 1080, "fo-FO", "The Faeroese language."),
        XmlMappedEnumMember("FARSI", 1065, "fa-IR", "The Farsi language."),
        XmlMappedEnumMember("FILIPINO", 1124, "fil-PH", "The Filipino language."),
        XmlMappedEnumMember("FINNISH", 1035, "fi-FI", "The Finnish language."),
        XmlMappedEnumMember(
            "FRANCH_CONGO_DRC", 9228, "fr-CD", "The French Congo DRC languag" "e."
        ),
        XmlMappedEnumMember("FRENCH", 1036, "fr-FR", "The French language."),
        XmlMappedEnumMember(
            "FRENCH_CAMEROON", 11276, "fr-CM", "The French Cameroon language" "."
        ),
        XmlMappedEnumMember(
            "FRENCH_CANADIAN", 3084, "fr-CA", "The French Canadian language."
        ),
        XmlMappedEnumMember(
            "FRENCH_COTED_IVOIRE",
            12300,
            "fr-CI",
            "The French Coted Ivoire " "language.",
        ),
        XmlMappedEnumMember(
            "FRENCH_HAITI", 15372, "fr-HT", "The French Haiti language."
        ),
        XmlMappedEnumMember(
            "FRENCH_LUXEMBOURG", 5132, "fr-LU", "The French Luxembourg langu" "age."
        ),
        XmlMappedEnumMember("FRENCH_MALI", 13324, "fr-ML", "The French Mali language."),
        XmlMappedEnumMember(
            "FRENCH_MONACO", 6156, "fr-MC", "The French Monaco language."
        ),
        XmlMappedEnumMember(
            "FRENCH_MOROCCO", 14348, "fr-MA", "The French Morocco language."
        ),
        XmlMappedEnumMember(
            "FRENCH_REUNION", 8204, "fr-RE", "The French Reunion language."
        ),
        XmlMappedEnumMember(
            "FRENCH_SENEGAL", 10252, "fr-SN", "The French Senegal language."
        ),
        XmlMappedEnumMember(
            "FRENCH_WEST_INDIES",
            7180,
            "fr-WINDIES",
            "The French West Indie" "s language.",
        ),
        XmlMappedEnumMember(
            "FRISIAN_NETHERLANDS", 1122, "fy-NL", "The Frisian Netherlands l" "anguage."
        ),
        XmlMappedEnumMember("FULFULDE", 1127, "ff-NG", "The Fulfulde language."),
        XmlMappedEnumMember(
            "GAELIC_IRELAND", 2108, "ga-IE", "The Gaelic Ireland language."
        ),
        XmlMappedEnumMember(
            "GAELIC_SCOTLAND", 1084, "en-US", "The Gaelic Scotland language."
        ),
        XmlMappedEnumMember("GALICIAN", 1110, "gl-ES", "The Galician language."),
        XmlMappedEnumMember("GEORGIAN", 1079, "ka-GE", "The Georgian language."),
        XmlMappedEnumMember("GERMAN", 1031, "de-DE", "The German language."),
        XmlMappedEnumMember(
            "GERMAN_AUSTRIA", 3079, "de-AT", "The German Austria language."
        ),
        XmlMappedEnumMember(
            "GERMAN_LIECHTENSTEIN",
            5127,
            "de-LI",
            "The German Liechtenstein" " language.",
        ),
        XmlMappedEnumMember(
            "GERMAN_LUXEMBOURG", 4103, "de-LU", "The German Luxembourg langu" "age."
        ),
        XmlMappedEnumMember("GREEK", 1032, "el-GR", "The Greek language."),
        XmlMappedEnumMember("GUARANI", 1140, "gn-PY", "The Guarani language."),
        XmlMappedEnumMember("GUJARATI", 1095, "gu-IN", "The Gujarati language."),
        XmlMappedEnumMember("HAUSA", 1128, "ha-NG", "The Hausa language."),
        XmlMappedEnumMember("HAWAIIAN", 1141, "haw-US", "The Hawaiian language."),
        XmlMappedEnumMember("HEBREW", 1037, "he-IL", "The Hebrew language."),
        XmlMappedEnumMember("HINDI", 1081, "hi-IN", "The Hindi language."),
        XmlMappedEnumMember("HUNGARIAN", 1038, "hu-HU", "The Hungarian language."),
        XmlMappedEnumMember("IBIBIO", 1129, "ibb-NG", "The Ibibio language."),
        XmlMappedEnumMember("ICELANDIC", 1039, "is-IS", "The Icelandic language."),
        XmlMappedEnumMember("IGBO", 1136, "ig-NG", "The Igbo language."),
        XmlMappedEnumMember("INDONESIAN", 1057, "id-ID", "The Indonesian language."),
        XmlMappedEnumMember("INUKTITUT", 1117, "iu-Cans-CA", "The Inuktitut language."),
        XmlMappedEnumMember("ITALIAN", 1040, "it-IT", "The Italian language."),
        XmlMappedEnumMember("JAPANESE", 1041, "ja-JP", "The Japanese language."),
        XmlMappedEnumMember("KANNADA", 1099, "kn-IN", "The Kannada language."),
        XmlMappedEnumMember("KANURI", 1137, "kr-NG", "The Kanuri language."),
        XmlMappedEnumMember("KASHMIRI", 1120, "ks-Arab", "The Kashmiri language."),
        XmlMappedEnumMember(
            "KASHMIRI_DEVANAGARI",
            2144,
            "ks-Deva",
            "The Kashmiri Devanagari" " language.",
        ),
        XmlMappedEnumMember("KAZAKH", 1087, "kk-KZ", "The Kazakh language."),
        XmlMappedEnumMember("KHMER", 1107, "kh-KH", "The Khmer language."),
        XmlMappedEnumMember("KIRGHIZ", 1088, "ky-KG", "The Kirghiz language."),
        XmlMappedEnumMember("KONKANI", 1111, "kok-IN", "The Konkani language."),
        XmlMappedEnumMember("KOREAN", 1042, "ko-KR", "The Korean language."),
        XmlMappedEnumMember("KYRGYZ", 1088, "ky-KG", "The Kyrgyz language."),
        XmlMappedEnumMember("LAO", 1108, "lo-LA", "The Lao language."),
        XmlMappedEnumMember("LATIN", 1142, "la-Latn", "The Latin language."),
        XmlMappedEnumMember("LATVIAN", 1062, "lv-LV", "The Latvian language."),
        XmlMappedEnumMember("LITHUANIAN", 1063, "lt-LT", "The Lithuanian language."),
        XmlMappedEnumMember(
            "MACEDONINAN_FYROM", 1071, "mk-MK", "The Macedonian FYROM langua" "ge."
        ),
        XmlMappedEnumMember(
            "MALAY_BRUNEI_DARUSSALAM",
            2110,
            "ms-BN",
            "The Malay Brunei Daru" "ssalam language.",
        ),
        XmlMappedEnumMember("MALAYALAM", 1100, "ml-IN", "The Malayalam language."),
        XmlMappedEnumMember("MALAYSIAN", 1086, "ms-MY", "The Malaysian language."),
        XmlMappedEnumMember("MALTESE", 1082, "mt-MT", "The Maltese language."),
        XmlMappedEnumMember("MANIPURI", 1112, "mni-IN", "The Manipuri language."),
        XmlMappedEnumMember("MAORI", 1153, "mi-NZ", "The Maori language."),
        XmlMappedEnumMember("MARATHI", 1102, "mr-IN", "The Marathi language."),
        XmlMappedEnumMember(
            "MEXICAN_SPANISH", 2058, "es-MX", "The Mexican Spanish language."
        ),
        XmlMappedEnumMember("MONGOLIAN", 1104, "mn-MN", "The Mongolian language."),
        XmlMappedEnumMember("NEPALI", 1121, "ne-NP", "The Nepali language."),
        XmlMappedEnumMember("NO_PROOFING", 1024, "en-US", "No proofing."),
        XmlMappedEnumMember(
            "NORWEGIAN_BOKMOL", 1044, "nb-NO", "The Norwegian Bokmol languag" "e."
        ),
        XmlMappedEnumMember(
            "NORWEGIAN_NYNORSK", 2068, "nn-NO", "The Norwegian Nynorsk langu" "age."
        ),
        XmlMappedEnumMember("ORIYA", 1096, "or-IN", "The Oriya language."),
        XmlMappedEnumMember("OROMO", 1138, "om-Ethi-ET", "The Oromo language."),
        XmlMappedEnumMember("PASHTO", 1123, "ps-AF", "The Pashto language."),
        XmlMappedEnumMember("POLISH", 1045, "pl-PL", "The Polish language."),
        XmlMappedEnumMember("PORTUGUESE", 2070, "pt-PT", "The Portuguese language."),
        XmlMappedEnumMember("PUNJABI", 1094, "pa-IN", "The Punjabi language."),
        XmlMappedEnumMember(
            "QUECHUA_BOLIVIA", 1131, "quz-BO", "The Quechua Bolivia language" "."
        ),
        XmlMappedEnumMember(
            "QUECHUA_ECUADOR", 2155, "quz-EC", "The Quechua Ecuador language" "."
        ),
        XmlMappedEnumMember(
            "QUECHUA_PERU", 3179, "quz-PE", "The Quechua Peru language."
        ),
        XmlMappedEnumMember(
            "RHAETO_ROMANIC", 1047, "rm-CH", "The Rhaeto Romanic language."
        ),
        XmlMappedEnumMember("ROMANIAN", 1048, "ro-RO", "The Romanian language."),
        XmlMappedEnumMember(
            "ROMANIAN_MOLDOVA", 2072, "ro-MO", "The Romanian Moldova languag" "e."
        ),
        XmlMappedEnumMember("RUSSIAN", 1049, "ru-RU", "The Russian language."),
        XmlMappedEnumMember(
            "RUSSIAN_MOLDOVA", 2073, "ru-MO", "The Russian Moldova language."
        ),
        XmlMappedEnumMember(
            "SAMI_LAPPISH", 1083, "se-NO", "The Sami Lappish language."
        ),
        XmlMappedEnumMember("SANSKRIT", 1103, "sa-IN", "The Sanskrit language."),
        XmlMappedEnumMember("SEPEDI", 1132, "ns-ZA", "The Sepedi language."),
        XmlMappedEnumMember(
            "SERBIAN_BOSNIA_HERZEGOVINA_CYRILLIC",
            7194,
            "sr-BA",
            "The Serbi" "an Bosnia Herzegovina Cyrillic language.",
        ),
        XmlMappedEnumMember(
            "SERBIAN_BOSNIA_HERZEGOVINA_LATIN",
            6170,
            "sr-Latn-BA",
            "The Ser" "bian Bosnia Herzegovina Latin language.",
        ),
        XmlMappedEnumMember(
            "SERBIAN_CYRILLIC", 3098, "sr-SP", "The Serbian Cyrillic languag" "e."
        ),
        XmlMappedEnumMember(
            "SERBIAN_LATIN", 2074, "sr-Latn-CS", "The Serbian Latin language" "."
        ),
        XmlMappedEnumMember("SESOTHO", 1072, "st-ZA", "The Sesotho language."),
        XmlMappedEnumMember(
            "SIMPLIFIED_CHINESE", 2052, "zh-CN", "The Simplified Chinese lan" "guage."
        ),
        XmlMappedEnumMember("SINDHI", 1113, "sd-Deva-IN", "The Sindhi language."),
        XmlMappedEnumMember(
            "SINDHI_PAKISTAN", 2137, "sd-Arab-PK", "The Sindhi Pakistan lang" "uage."
        ),
        XmlMappedEnumMember("SINHALESE", 1115, "si-LK", "The Sinhalese language."),
        XmlMappedEnumMember("SLOVAK", 1051, "sk-SK", "The Slovak language."),
        XmlMappedEnumMember("SLOVENIAN", 1060, "sl-SI", "The Slovenian language."),
        XmlMappedEnumMember("SOMALI", 1143, "so-SO", "The Somali language."),
        XmlMappedEnumMember("SORBIAN", 1070, "wen-DE", "The Sorbian language."),
        XmlMappedEnumMember("SPANISH", 1034, "es-ES_tradnl", "The Spanish language."),
        XmlMappedEnumMember(
            "SPANISH_ARGENTINA", 11274, "es-AR", "The Spanish Argentina lang" "uage."
        ),
        XmlMappedEnumMember(
            "SPANISH_BOLIVIA", 16394, "es-BO", "The Spanish Bolivia language" "."
        ),
        XmlMappedEnumMember(
            "SPANISH_CHILE", 13322, "es-CL", "The Spanish Chile language."
        ),
        XmlMappedEnumMember(
            "SPANISH_COLOMBIA", 9226, "es-CO", "The Spanish Colombia languag" "e."
        ),
        XmlMappedEnumMember(
            "SPANISH_COSTA_RICA", 5130, "es-CR", "The Spanish Costa Rica lan" "guage."
        ),
        XmlMappedEnumMember(
            "SPANISH_DOMINICAN_REPUBLIC",
            7178,
            "es-DO",
            "The Spanish Domini" "can Republic language.",
        ),
        XmlMappedEnumMember(
            "SPANISH_ECUADOR", 12298, "es-EC", "The Spanish Ecuador language" "."
        ),
        XmlMappedEnumMember(
            "SPANISH_EL_SALVADOR",
            17418,
            "es-SV",
            "The Spanish El Salvador " "language.",
        ),
        XmlMappedEnumMember(
            "SPANISH_GUATEMALA", 4106, "es-GT", "The Spanish Guatemala langu" "age."
        ),
        XmlMappedEnumMember(
            "SPANISH_HONDURAS", 18442, "es-HN", "The Spanish Honduras langua" "ge."
        ),
        XmlMappedEnumMember(
            "SPANISH_MODERN_SORT", 3082, "es-ES", "The Spanish Modern Sort l" "anguage."
        ),
        XmlMappedEnumMember(
            "SPANISH_NICARAGUA", 19466, "es-NI", "The Spanish Nicaragua lang" "uage."
        ),
        XmlMappedEnumMember(
            "SPANISH_PANAMA", 6154, "es-PA", "The Spanish Panama language."
        ),
        XmlMappedEnumMember(
            "SPANISH_PARAGUAY", 15370, "es-PY", "The Spanish Paraguay langua" "ge."
        ),
        XmlMappedEnumMember(
            "SPANISH_PERU", 10250, "es-PE", "The Spanish Peru language."
        ),
        XmlMappedEnumMember(
            "SPANISH_PUERTO_RICO",
            20490,
            "es-PR",
            "The Spanish Puerto Rico " "language.",
        ),
        XmlMappedEnumMember(
            "SPANISH_URUGUAY", 14346, "es-UR", "The Spanish Uruguay language" "."
        ),
        XmlMappedEnumMember(
            "SPANISH_VENEZUELA", 8202, "es-VE", "The Spanish Venezuela langu" "age."
        ),
        XmlMappedEnumMember("SUTU", 1072, "st-ZA", "The Sutu language."),
        XmlMappedEnumMember("SWAHILI", 1089, "sw-KE", "The Swahili language."),
        XmlMappedEnumMember("SWEDISH", 1053, "sv-SE", "The Swedish language."),
        XmlMappedEnumMember(
            "SWEDISH_FINLAND", 2077, "sv-FI", "The Swedish Finland language."
        ),
        XmlMappedEnumMember(
            "SWISS_FRENCH", 4108, "fr-CH", "The Swiss French language."
        ),
        XmlMappedEnumMember(
            "SWISS_GERMAN", 2055, "de-CH", "The Swiss German language."
        ),
        XmlMappedEnumMember(
            "SWISS_ITALIAN", 2064, "it-CH", "The Swiss Italian language."
        ),
        XmlMappedEnumMember("SYRIAC", 1114, "syr-SY", "The Syriac language."),
        XmlMappedEnumMember("TAJIK", 1064, "tg-TJ", "The Tajik language."),
        XmlMappedEnumMember(
            "TAMAZIGHT", 1119, "tzm-Arab-MA", "The Tamazight language."
        ),
        XmlMappedEnumMember(
            "TAMAZIGHT_LATIN", 2143, "tmz-DZ", "The Tamazight Latin language" "."
        ),
        XmlMappedEnumMember("TAMIL", 1097, "ta-IN", "The Tamil language."),
        XmlMappedEnumMember("TATAR", 1092, "tt-RU", "The Tatar language."),
        XmlMappedEnumMember("TELUGU", 1098, "te-IN", "The Telugu language."),
        XmlMappedEnumMember("THAI", 1054, "th-TH", "The Thai language."),
        XmlMappedEnumMember("TIBETAN", 1105, "bo-CN", "The Tibetan language."),
        XmlMappedEnumMember(
            "TIGRIGNA_ERITREA", 2163, "ti-ER", "The Tigrigna Eritrea languag" "e."
        ),
        XmlMappedEnumMember(
            "TIGRIGNA_ETHIOPIC", 1139, "ti-ET", "The Tigrigna Ethiopic langu" "age."
        ),
        XmlMappedEnumMember(
            "TRADITIONAL_CHINESE", 1028, "zh-TW", "The Traditional Chinese l" "anguage."
        ),
        XmlMappedEnumMember("TSONGA", 1073, "ts-ZA", "The Tsonga language."),
        XmlMappedEnumMember("TSWANA", 1074, "tn-ZA", "The Tswana language."),
        XmlMappedEnumMember("TURKISH", 1055, "tr-TR", "The Turkish language."),
        XmlMappedEnumMember("TURKMEN", 1090, "tk-TM", "The Turkmen language."),
        XmlMappedEnumMember("UKRAINIAN", 1058, "uk-UA", "The Ukrainian language."),
        XmlMappedEnumMember("URDU", 1056, "ur-PK", "The Urdu language."),
        XmlMappedEnumMember(
            "UZBEK_CYRILLIC", 2115, "uz-UZ", "The Uzbek Cyrillic language."
        ),
        XmlMappedEnumMember(
            "UZBEK_LATIN", 1091, "uz-Latn-UZ", "The Uzbek Latin language."
        ),
        XmlMappedEnumMember("VENDA", 1075, "ve-ZA", "The Venda language."),
        XmlMappedEnumMember("VIETNAMESE", 1066, "vi-VN", "The Vietnamese language."),
        XmlMappedEnumMember("WELSH", 1106, "cy-GB", "The Welsh language."),
        XmlMappedEnumMember("XHOSA", 1076, "xh-ZA", "The Xhosa language."),
        XmlMappedEnumMember("YI", 1144, "ii-CN", "The Yi language."),
        XmlMappedEnumMember("YIDDISH", 1085, "yi-Hebr", "The Yiddish language."),
        XmlMappedEnumMember("YORUBA", 1130, "yo-NG", "The Yoruba language."),
        XmlMappedEnumMember("ZULU", 1077, "zu-ZA", "The Zulu language."),
    )



# encoding: utf-8

"""
Test suite for pptx.chart.legend module
"""

from __future__ import absolute_import, print_function

import pytest

from pptx.chart.legend import Legend
from pptx.enum.chart import XL_LEGEND_POSITION
from pptx.text.text import Font

from ..unitutil.cxml import element, xml


class DescribeLegend(object):
    def it_provides_access_to_its_font(self, font_fixture):
        legend, expected_xml = font_fixture
        font = legend.font
        assert legend._element.xml == expected_xml
        assert isinstance(font, Font)
        assert font._element == legend._element.xpath(".//a:defRPr")[0]

    def it_knows_its_horizontal_offset(self, horz_offset_get_fixture):
        legend, expected_value = horz_offset_get_fixture
        assert legend.horz_offset == expected_value

    def it_can_change_its_horizontal_offset(self, horz_offset_set_fixture):
        legend, new_value, expected_xml = horz_offset_set_fixture
        legend.horz_offset = new_value
        assert legend._element.xml == expected_xml

    def it_knows_whether_it_should_overlap_the_chart(
        self, include_in_layout_get_fixture
    ):
        legend, expected_value = include_in_layout_get_fixture
        assert legend.include_in_layout == expected_value

    def it_can_change_whether_it_overlaps_the_chart(
        self, include_in_layout_set_fixture
    ):
        legend, new_value, expected_xml = include_in_layout_set_fixture
        legend.include_in_layout = new_value
        assert legend._element.xml == expected_xml

    def it_knows_its_position_with_respect_to_the_chart(self, position_get_fixture):
        legend, expected_value = position_get_fixture
        assert legend.position == expected_value

    def it_can_change_its_position(self, position_set_fixture):
        legend, new_value, expected_xml = position_set_fixture
        legend.position = new_value
        assert legend._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            (
                "c:legend{a:b=c}",
                "c:legend{a:b=c}/c:txPr/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr)",
            )
        ]
    )
    def font_fixture(self, request):
        legend_cxml, expected_cxml = request.param
        legend = Legend(element(legend_cxml))
        expected_xml = xml(expected_cxml)
        return legend, expected_xml

    @pytest.fixture(
        params=[
            ("c:legend", 0.0),
            ("c:legend/c:layout", 0.0),
            ("c:legend/c:layout/c:manualLayout", 0.0),
            ("c:legend/c:layout/c:manualLayout/c:xMode", 0.0),
            ("c:legend/c:layout/c:manualLayout/c:xMode{val=edge}", 0.0),
            ("c:legend/c:layout/c:manualLayout/c:xMode{val=factor}", 0.0),
            ("c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=0.42})", 0.42),
            (
                "c:legend/c:layout/c:manualLayout/(c:xMode{val=factor},c:x{val=0.42"
                "})",
                0.42,
            ),
        ]
    )
    def horz_offset_get_fixture(self, request):
        legend_cxml, expected_value = request.param
        legend = Legend(element(legend_cxml))
        return legend, expected_value

    @pytest.fixture(
        params=[
            (
                "c:legend",
                0.42,
                "c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=0.42})",
            ),
            (
                "c:legend/c:layout",
                0.42,
                "c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=0.42})",
            ),
            (
                "c:legend/c:layout/c:manualLayout",
                -0.42,
                "c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=-0.42})",
            ),
            (
                "c:legend/c:layout/c:manualLayout/c:xMode{val=edge}",
                -0.1,
                "c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=-0.1})",
            ),
            (
                "c:legend/c:layout/c:manualLayout/c:xMode{val=factor}",
                0.2,
                "c:legend/c:layout/c:manualLayout/(c:xMode,c:x{val=0.2})",
            ),
            ("c:legend/c:layout/c:manualLayout", 0, "c:legend/c:layout"),
        ]
    )
    def horz_offset_set_fixture(self, request):
        legend_cxml, new_value, expected_legend_cxml = request.param
        legend = Legend(element(legend_cxml))
        expected_xml = xml(expected_legend_cxml)
        return legend, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:legend", True),
            ("c:legend/c:overlay", True),
            ("c:legend/c:overlay{val=1}", True),
            ("c:legend/c:overlay{val=0}", False),
        ]
    )
    def include_in_layout_get_fixture(self, request):
        legend_cxml, expected_value = request.param
        legend = Legend(element(legend_cxml))
        return legend, expected_value

    @pytest.fixture(
        params=[
            ("c:legend", True, "c:legend/c:overlay{val=1}"),
            ("c:legend", False, "c:legend/c:overlay{val=0}"),
            ("c:legend/c:overlay{val=0}", True, "c:legend/c:overlay{val=1}"),
            ("c:legend/c:overlay{val=1}", False, "c:legend/c:overlay{val=0}"),
            ("c:legend/c:overlay{val=1}", None, "c:legend"),
        ]
    )
    def include_in_layout_set_fixture(self, request):
        legend_cxml, new_value, expected_legend_cxml = request.param
        legend = Legend(element(legend_cxml))
        expected_xml = xml(expected_legend_cxml)
        return legend, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:legend", "RIGHT"),
            ("c:legend/c:legendPos", "RIGHT"),
            ("c:legend/c:legendPos{val=r}", "RIGHT"),
            ("c:legend/c:legendPos{val=b}", "BOTTOM"),
        ]
    )
    def position_get_fixture(self, request):
        legend_cxml, expected_enum_member = request.param
        legend = Legend(element(legend_cxml))
        expected_value = getattr(XL_LEGEND_POSITION, expected_enum_member)
        return legend, expected_value

    @pytest.fixture(
        params=[
            ("c:legend/c:legendPos{val=r}", "BOTTOM", "c:legend/c:legendPos{val=b}"),
            ("c:legend/c:legendPos{val=b}", "RIGHT", "c:legend/c:legendPos"),
            ("c:legend", "TOP", "c:legend/c:legendPos{val=t}"),
            ("c:legend/c:legendPos", "CORNER", "c:legend/c:legendPos{val=tr}"),
        ]
    )
    def position_set_fixture(self, request):
        legend_cxml, new_enum_member, expected_legend_cxml = request.param
        legend = Legend(element(legend_cxml))
        new_value = getattr(XL_LEGEND_POSITION, new_enum_member)
        expected_xml = xml(expected_legend_cxml)
        return legend, new_value, expected_xml
